@@ -28,12 +28,7 @@ insert_breakpoint(struct process *proc, void *addr,
 		  struct library_symbol *libsym)
 {
 	struct breakpoint *sbp;
-
-	if (!proc->breakpoints) {
-		proc->breakpoints =
-		    dict_init(dict_key2hash_int, dict_key_cmp_int);
-		/* atexit(brk_dict_clear); *//* why bother to do this on exit? */
-	}
+	debug(1, "symbol=%s, addr=%p", libsym?libsym->name:"(nil)", addr);
 
 	if (!addr)
 		return;
@@ -53,6 +48,10 @@ insert_breakpoint(struct process *proc, void *addr,
 		if (libsym)
 			libsym->brkpnt = sbp;
 	}
+#ifdef __arm__
+	sbp->thumb_mode = proc->thumb_mode;
+	proc->thumb_mode = 0;
+#endif
 	sbp->enabled++;
 	if (sbp->enabled == 1 && proc->pid)
 		enable_breakpoint(proc->pid, sbp);
@@ -104,6 +103,34 @@ void enable_all_breakpoints(struct process *proc)
 			dict_apply_to_all(proc->breakpoints, enable_bp_cb,
 					  proc);
 		}
+#ifdef __mips__
+		{
+			// I'm sure there is a nicer way to do this. We need to
+			// insert breakpoints _after_ the child has been started.
+			struct library_symbol *sym;
+			struct library_symbol *new_sym;
+			sym=proc->list_of_symbols;
+			while(sym){
+				void *addr= sym2addr(proc,sym);
+				if(!addr){
+					sym=sym->next;
+					continue;
+				}
+				if(dict_find_entry(proc->breakpoints,addr)){
+					sym=sym->next;
+					continue;
+				}
+				debug(2,"inserting bp %p %s",addr,sym->name);
+				new_sym=malloc(sizeof(*new_sym));
+				memcpy(new_sym,sym,sizeof(*new_sym));
+				new_sym->next=proc->list_of_symbols;
+				proc->list_of_symbols=new_sym;
+				new_sym->brkpnt=0;
+				insert_breakpoint(proc, addr, new_sym);
+				sym=sym->next;
+			}
+		}
+#endif
 	}
 	proc->breakpoints_enabled = 1;
 }
@@ -135,11 +162,11 @@ void breakpoints_init(struct process *proc)
 	struct library_symbol *sym;
 
 	if (proc->breakpoints) {	/* let's remove that struct */
-		/* TODO FIXME XXX: free() all "struct breakpoint"s */
 		dict_apply_to_all(proc->breakpoints, free_bp_cb, NULL);
 		dict_clear(proc->breakpoints);
 		proc->breakpoints = NULL;
 	}
+	proc->breakpoints = dict_init(dict_key2hash_int, dict_key_cmp_int);
 
 	if (opt_L && proc->filename) {
 		proc->list_of_symbols = read_elf(proc);
@@ -165,11 +192,9 @@ void breakpoints_init(struct process *proc)
 	} else {
 		proc->list_of_symbols = NULL;
 	}
-	sym = proc->list_of_symbols;
-	while (sym) {
+	for (sym = proc->list_of_symbols; sym; sym = sym->next) {
 		/* proc->pid==0 delays enabling. */
 		insert_breakpoint(proc, sym2addr(proc, sym), sym);
-		sym = sym->next;
 	}
 	proc->callstack_depth = 0;
 	proc->breakpoints_enabled = -1;
