@@ -24,7 +24,13 @@ struct ltelf {
 	GElf_Addr *opd_addr;
 	size_t opd_size;
 	Elf32_Word *hash;
+	int hash_type;
 	int lte_flags;
+#ifdef __mips__
+	size_t pltgot_addr;
+	size_t mips_local_gotno;
+	size_t mips_gotsym;
+#endif // __mips__
 };
 
 #define LTE_HASH_MALLOCED 1
@@ -39,4 +45,7 @@ extern struct library_symbol *read_elf(struct process *);
 
 extern GElf_Addr arch_plt_sym_val(struct ltelf *, size_t, GElf_Rela *);
 
+#ifndef SHT_GNU_HASH
+#define SHT_GNU_HASH	0x6ffffff6	/* GNU-style hash table. */
+#endif
 #endif
