@@ -26,6 +26,9 @@ struct breakpoint {
 	unsigned char orig_value[BREAKPOINT_LENGTH];
 	int enabled;
 	struct library_symbol *libsym;
+#ifdef __arm__
+	int thumb_mode;
+#endif
 };
 
 enum arg_type {
@@ -37,31 +40,82 @@ enum arg_type {
 	ARGTYPE_ULONG,
 	ARGTYPE_OCTAL,
 	ARGTYPE_CHAR,
+	ARGTYPE_SHORT,
+	ARGTYPE_USHORT,
+	ARGTYPE_FLOAT,		/* float value, may require index */
+	ARGTYPE_DOUBLE,		/* double value, may require index */
 	ARGTYPE_ADDR,
 	ARGTYPE_FILE,
 	ARGTYPE_FORMAT,		/* printf-like format */
-	ARGTYPE_STRING,
-	ARGTYPE_STRING0,	/* stringN: string up to (arg N) bytes */
-	ARGTYPE_STRING1,
-	ARGTYPE_STRING2,
-	ARGTYPE_STRING3,
-	ARGTYPE_STRING4,
-	ARGTYPE_STRING5
+	ARGTYPE_STRING,		/* NUL-terminated string */
+	ARGTYPE_STRING_N,	/* String of known maxlen */
+	ARGTYPE_ARRAY,		/* Series of values in memory */
+	ARGTYPE_ENUM,		/* Enumeration */
+	ARGTYPE_STRUCT,		/* Structure of values */
+	ARGTYPE_POINTER,	/* Pointer to some other type */
+	ARGTYPE_COUNT		/* number of ARGTYPE_* values */
 };
 
+typedef struct arg_type_info_t {
+	enum arg_type type;
+	union {
+		// ARGTYPE_ENUM
+		struct {
+			size_t entries;
+			char **keys;
+			int *values;
+		} enum_info;
+
+		// ARGTYPE_ARRAY
+		struct {
+			struct arg_type_info_t *elt_type;
+			size_t elt_size;
+			int len_spec;
+		} array_info;
+
+		// ARGTYPE_STRING_N
+		struct {
+			int size_spec;
+		} string_n_info;
+
+		// ARGTYPE_STRUCT
+		struct {
+			struct arg_type_info_t **fields;	// NULL-terminated
+			size_t *offset;
+			size_t size;
+		} struct_info;
+
+		// ARGTYPE_POINTER
+		struct {
+			struct arg_type_info_t *info;
+		} ptr_info;
+
+		// ARGTYPE_FLOAT
+		struct {
+			size_t float_index;
+		} float_info;
+
+		// ARGTYPE_DOUBLE
+		struct {
+			size_t float_index;
+		} double_info;
+	} u;
+} arg_type_info;
+
 enum tof {
 	LT_TOF_NONE = 0,
 	LT_TOF_FUNCTION,	/* A real library function */
 	LT_TOF_FUNCTIONR,	/* Return from a real library function */
 	LT_TOF_SYSCALL,		/* A syscall */
-	LT_TOF_SYSCALLR		/* Return from a syscall */
+	LT_TOF_SYSCALLR,	/* Return from a syscall */
+	LT_TOF_STRUCT		/* Not a function; read args from struct */
 };
 
 struct function {
 	const char *name;
-	enum arg_type return_type;
+	arg_type_info *return_info;
 	int num_params;
-	enum arg_type arg_types[MAX_ARGS];
+	arg_type_info *arg_info[MAX_ARGS];
 	int params_right;
 	struct function *next;
 };
@@ -119,6 +173,9 @@ struct process {
 	void *arch_ptr;
 	short e_machine;
 	short need_to_reinitialize_breakpoints;
+#ifdef __arm__
+	int thumb_mode; /* ARM execution mode: 0: ARM mode, 1: Thumb mode */
+#endif
 
 	/* output: */
 	enum tof type_being_displayed;
@@ -136,12 +193,14 @@ struct event {
 		LT_EV_EXIT_SIGNAL,
 		LT_EV_SYSCALL,
 		LT_EV_SYSRET,
+		LT_EV_ARCH_SYSCALL,
+		LT_EV_ARCH_SYSRET,
 		LT_EV_BREAKPOINT
 	} thing;
 	union {
 		int ret_val;	/* _EV_EXIT */
 		int signum;	/* _EV_SIGNAL, _EV_EXIT_SIGNAL */
-		int sysnum;	/* _EV_SYSCALL, _EV_SYSRET */
+		int sysnum;	/* _EV_SYSCALL, _EV_SYSRET, _EV_ARCH_SYSCALL, _EV_ARCH_SYSRET */
 		void *brk_addr;	/* _EV_BREAKPOINT */
 	} e_un;
 };
@@ -159,8 +218,7 @@ extern void *instruction_pointer;
 extern struct event *wait_for_something(void);
 extern void process_event(struct event *event);
 extern void execute_program(struct process *, char **);
-extern int display_arg(enum tof type, struct process *proc, int arg_num,
-		       enum arg_type at);
+extern int display_arg(enum tof type, struct process *proc, int arg_num, arg_type_info *info);
 extern struct breakpoint *address2bpstruct(struct process *proc, void *addr);
 extern void breakpoints_init(struct process *proc);
 extern void insert_breakpoint(struct process *proc, void *addr,
@@ -173,6 +231,7 @@ extern void reinitialize_breakpoints(struct process *);
 extern struct process *open_program(char *filename, pid_t pid);
 extern void open_pid(pid_t pid, int verbose);
 extern void show_summary(void);
+extern arg_type_info *lookup_prototype(enum arg_type at);
 
 /* Arch-dependent stuff: */
 extern char *pid2name(pid_t pid);
@@ -189,15 +248,17 @@ extern void enable_breakpoint(pid_t pid, struct breakpoint *sbp);
 extern void disable_breakpoint(pid_t pid, const struct breakpoint *sbp);
 extern int fork_p(struct process *proc, int sysnum);
 extern int exec_p(struct process *proc, int sysnum);
+extern int was_exec(struct process *proc, int status);
 extern int syscall_p(struct process *proc, int status, int *sysnum);
 extern void continue_process(pid_t pid);
 extern void continue_after_signal(pid_t pid, int signum);
 extern void continue_after_breakpoint(struct process *proc,
 				      struct breakpoint *sbp);
 extern void continue_enabling_breakpoint(pid_t pid, struct breakpoint *sbp);
-extern long gimme_arg(enum tof type, struct process *proc, int arg_num);
+extern long gimme_arg(enum tof type, struct process *proc, int arg_num, arg_type_info *info);
 extern void save_register_args(enum tof type, struct process *proc);
 extern int umovestr(struct process *proc, void *addr, int len, void *laddr);
+extern int umovelong (struct process *proc, void *addr, long *result, arg_type_info *info);
 extern int ffcheck(void *maddr);
 extern void *sym2addr(struct process *, struct library_symbol *);
 
