@@ -24,7 +24,9 @@ static void process_signal(struct event *event);
 static void process_exit(struct event *event);
 static void process_exit_signal(struct event *event);
 static void process_syscall(struct event *event);
+static void process_arch_syscall(struct event *event);
 static void process_sysret(struct event *event);
+static void process_arch_sysret(struct event *event);
 static void process_breakpoint(struct event *event);
 static void remove_proc(struct process *proc);
 
@@ -81,6 +83,24 @@ static char *sysname(struct process *proc, int sysnum)
 	}
 }
 
+static char *arch_sysname(struct process *proc, int sysnum)
+{
+	static char result[128];
+	static char *arch_syscalent[] = {
+#include "arch_syscallent.h"
+	};
+	int nsyscals = sizeof arch_syscalent / sizeof arch_syscalent[0];
+
+	if (sysnum < 0 || sysnum >= nsyscals) {
+		sprintf(result, "ARCH_%d", sysnum);
+		return result;
+	} else {
+		sprintf(result, "ARCH_%s",
+				arch_syscalent[sysnum]);
+		return result;
+	}
+}
+
 void process_event(struct event *event)
 {
 	switch (event->thing) {
@@ -115,6 +135,18 @@ void process_event(struct event *event)
 		      event->e_un.sysnum);
 		process_sysret(event);
 		return;
+	case LT_EV_ARCH_SYSCALL:
+		debug(1, "event: arch_syscall (%s [%d])",
+				arch_sysname(event->proc, event->e_un.sysnum),
+				event->e_un.sysnum);
+		process_arch_syscall(event);
+		return;
+	case LT_EV_ARCH_SYSRET:
+		debug(1, "event: arch_sysret (%s [%d])",
+				arch_sysname(event->proc, event->e_un.sysnum),
+				event->e_un.sysnum);
+		process_arch_sysret(event);
+		return;
 	case LT_EV_BREAKPOINT:
 		debug(1, "event: breakpoint");
 		process_breakpoint(event);
@@ -185,8 +217,7 @@ static void process_syscall(struct event *event)
 		output_left(LT_TOF_SYSCALL, event->proc,
 			    sysname(event->proc, event->e_un.sysnum));
 	}
-	if (fork_p(event->proc, event->e_un.sysnum)
-	    || exec_p(event->proc, event->e_un.sysnum)) {
+	if (fork_p(event->proc, event->e_un.sysnum)) {
 		disable_all_breakpoints(event->proc);
 	} else if (event->proc->breakpoints_enabled == 0) {
 		enable_all_breakpoints(event->proc);
@@ -195,6 +226,19 @@ static void process_syscall(struct event *event)
 	continue_process(event->proc->pid);
 }
 
+static void process_arch_syscall(struct event *event)
+{
+	if (opt_S) {
+		output_left(LT_TOF_SYSCALL, event->proc,
+				arch_sysname(event->proc, event->e_un.sysnum));
+	}
+	if (event->proc->breakpoints_enabled == 0) {
+		enable_all_breakpoints(event->proc);
+	}
+	callstack_push_syscall(event->proc, 0xf0000 + event->e_un.sysnum);
+	continue_process(event->proc->pid);
+}
+
 struct timeval current_time_spent;
 
 static void calc_time_spent(struct process *proc)
@@ -225,8 +269,10 @@ static void process_sysret(struct event *event)
 	}
 	if (fork_p(event->proc, event->e_un.sysnum)) {
 		if (opt_f) {
+			arg_type_info info;
+			info.type = ARGTYPE_LONG;
 			pid_t child =
-			    gimme_arg(LT_TOF_SYSCALLR, event->proc, -1);
+			    gimme_arg(LT_TOF_SYSCALLR, event->proc, -1, &info);
 			if (child > 0) {
 				open_pid(child, 0);
 			}
@@ -238,21 +284,18 @@ static void process_sysret(struct event *event)
 		output_right(LT_TOF_SYSCALLR, event->proc,
 			     sysname(event->proc, event->e_un.sysnum));
 	}
-	if (exec_p(event->proc, event->e_un.sysnum)) {
-		if (gimme_arg(LT_TOF_SYSCALLR, event->proc, -1) == 0) {
-			pid_t saved_pid;
-			event->proc->mask_32bit = 0;
-			event->proc->personality = 0;
-			/* FIXME: Leak, should have arch_dep_free.
-			   But we are leaking here much more than that.  */
-			event->proc->arch_ptr = NULL;
-			event->proc->filename = pid2name(event->proc->pid);
-			saved_pid = event->proc->pid;
-			event->proc->pid = 0;
-			breakpoints_init(event->proc);
-			event->proc->pid = saved_pid;
-		} else
-			enable_all_breakpoints(event->proc);
+	continue_process(event->proc->pid);
+}
+
+static void process_arch_sysret(struct event *event)
+{
+	if (opt_T || opt_c) {
+		calc_time_spent(event->proc);
+	}
+	callstack_pop(event->proc);
+	if (opt_S) {
+		output_right(LT_TOF_SYSCALLR, event->proc,
+				arch_sysname(event->proc, event->e_un.sysnum));
 	}
 	continue_process(event->proc->pid);
 }
@@ -260,23 +303,32 @@ static void process_sysret(struct event *event)
 static void process_breakpoint(struct event *event)
 {
 	int i, j;
-	struct breakpoint *sbp, *nxtbp;
+	struct breakpoint *sbp;
 
 	debug(2, "event: breakpoint (%p)", event->e_un.brk_addr);
-	if ((sbp = event->proc->breakpoint_being_enabled) != 0) {
+
 #ifdef __powerpc__
-		char nop_inst[] = PPC_NOP;
-                if (memcmp(sbp->orig_value, nop_inst, PPC_NOP_LENGTH) == 0) {
-                	nxtbp = address2bpstruct(event->proc,
-                                                 event->e_un.brk_addr +
-                                                 	PPC_NOP_LENGTH);
-			if (nxtbp != 0) {
-				enable_breakpoint(event->proc->pid, sbp);
-				continue_after_breakpoint(event->proc, nxtbp);
+	/* Need to skip following NOP's to prevent a fake function from being stacked.  */
+	long stub_addr = (long) get_count_register(event->proc);
+	struct breakpoint *stub_bp = NULL;
+	char nop_instruction[] = PPC_NOP;
+
+	stub_bp = address2bpstruct (event->proc, event->e_un.brk_addr);
+
+	if (stub_bp) {
+		unsigned char *bp_instruction = stub_bp->orig_value;
+
+		if (memcmp(bp_instruction, nop_instruction,
+			    PPC_NOP_LENGTH) == 0) {
+			if (stub_addr != (long) event->e_un.brk_addr) {
+				set_instruction_pointer (event->proc, event->e_un.brk_addr + 4);
+				continue_process(event->proc->pid);
 				return;
 			}
 		}
+	}
 #endif
+	if ((sbp = event->proc->breakpoint_being_enabled) != 0) {
 		/* Reinsert breakpoint */
 		continue_enabling_breakpoint(event->proc->pid,
 					     event->proc->
@@ -320,6 +372,23 @@ static void process_breakpoint(struct event *event)
 							  libsym);
 				}
 			}
+#elif defined(__mips__)
+			void *addr;
+			void *old_addr;
+			struct library_symbol *sym= event->proc->callstack[i].c_un.libfunc;
+			assert(sym && sym->brkpnt);
+			old_addr=sym->brkpnt->addr;
+			addr=sym2addr(event->proc,sym);
+			assert(old_addr !=0 && addr !=0);
+			if(addr != old_addr){
+				struct library_symbol *new_sym;
+				new_sym=malloc(sizeof(*new_sym));
+				memcpy(new_sym,sym,sizeof(*new_sym));
+				new_sym->next=event->proc->list_of_symbols;
+				event->proc->list_of_symbols=new_sym;
+				new_sym->brkpnt=0;
+				insert_breakpoint(event->proc, addr, new_sym);
+			}
 #endif
 			for (j = event->proc->callstack_depth - 1; j > i; j--) {
 				callstack_pop(event->proc);
@@ -400,7 +469,7 @@ callstack_push_symfunc(struct process *proc, struct library_symbol *sym)
 	elem->c_un.libfunc = sym;
 
 	elem->return_addr = proc->return_addr;
-        if (elem->return_addr) {
+	if (elem->return_addr) {
 		insert_breakpoint(proc, elem->return_addr, 0);
 	}
 
