@@ -25,6 +25,7 @@ struct event *wait_for_something(void)
 	pid_t pid;
 	int status;
 	int tmp;
+	int stop_signal;
 
 	if (!list_of_processes) {
 		debug(1, "No more children");
@@ -63,14 +64,26 @@ struct event *wait_for_something(void)
 		    get_instruction_pointer(event.proc);
 	}
 	switch (syscall_p(event.proc, status, &tmp)) {
-	case 1:
-		event.thing = LT_EV_SYSCALL;
-		event.e_un.sysnum = tmp;
-		return &event;
-	case 2:
-		event.thing = LT_EV_SYSRET;
-		event.e_un.sysnum = tmp;
-		return &event;
+		case 1:
+			event.thing = LT_EV_SYSCALL;
+			event.e_un.sysnum = tmp;
+			return &event;
+		case 2:
+			event.thing = LT_EV_SYSRET;
+			event.e_un.sysnum = tmp;
+			return &event;
+		case 3:
+			event.thing = LT_EV_ARCH_SYSCALL;
+			event.e_un.sysnum = tmp;
+			return &event;
+		case 4:
+			event.thing = LT_EV_ARCH_SYSRET;
+			event.e_un.sysnum = tmp;
+			return &event;
+		case -1:
+			event.thing = LT_EV_NONE;
+			continue_process(event.proc->pid);
+			return &event;
 	}
 	if (WIFEXITED(status)) {
 		event.thing = LT_EV_EXIT;
@@ -86,12 +99,52 @@ struct event *wait_for_something(void)
 		event.thing = LT_EV_UNKNOWN;
 		return &event;
 	}
-	if ((WSTOPSIG(status) != (SIGTRAP | event.proc->tracesysgood)) &&
-	    (WSTOPSIG(status) != SIGTRAP)) {
+
+	stop_signal = WSTOPSIG(status);
+
+	/* On some targets, breakpoints are signalled not using
+	   SIGTRAP, but also with SIGILL, SIGSEGV or SIGEMT.  Check
+	   for these. */
+	if (stop_signal == SIGSEGV
+	    || stop_signal == SIGILL
+#ifdef SIGEMT
+	    || stop_signal == SIGEMT
+#endif
+	    ) {
+		// If we didn't need to know IP so far, get it now.
+		void * addr = opt_i
+		  ? event.proc->instruction_pointer
+		  : (event.proc->instruction_pointer = get_instruction_pointer (event.proc));
+
+		if (address2bpstruct(event.proc, addr))
+			stop_signal = SIGTRAP;
+	}
+
+	if (stop_signal != (SIGTRAP | event.proc->tracesysgood)
+	    && stop_signal != SIGTRAP) {
 		event.thing = LT_EV_SIGNAL;
-		event.e_un.signum = WSTOPSIG(status);
+		event.e_un.signum = stop_signal;
 		return &event;
 	}
+
+	if (was_exec(event.proc, status)) {
+		pid_t saved_pid;
+
+		event.thing = LT_EV_NONE;
+		event.e_un.signum = WSTOPSIG(status);
+		debug(1, "Placing breakpoints for the new program");
+		event.proc->mask_32bit = 0;
+		event.proc->personality = 0;
+		event.proc->arch_ptr = NULL;
+		event.proc->filename = pid2name(event.proc->pid);
+		saved_pid = event.proc->pid;
+		event.proc->pid = 0;
+		breakpoints_init(event.proc);
+		event.proc->pid = saved_pid;
+		continue_process(event.proc->pid);
+		return &event;
+	}
+
 	event.thing = LT_EV_BREAKPOINT;
 	if (!event.proc->instruction_pointer) {
 		event.proc->instruction_pointer =
